"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createControllerDependencies = void 0;
const JSONataService_1 = __importDefault(require("../services/JSONataService"));
const NodeRedContextService_1 = __importDefault(require("../services/NodeRedContextService"));
const TypedInputService_1 = __importDefault(require("../services/TypedInputService"));
/**
 * Create some of the dependencies needed for a BaseNode controller.
 * @param node The node to create the controller for.
 * @param homeAssistant The home assistant instance to use.
 * @returns The dependencies needed for a BaseNode controller.
 */
function createControllerDependencies(node, homeAssistant) {
    const jsonataService = new JSONataService_1.default({
        homeAssistant,
        node,
    });
    const nodeRedContextService = new NodeRedContextService_1.default(node);
    const typedInputService = new TypedInputService_1.default({
        nodeConfig: node.config,
        context: nodeRedContextService,
        jsonata: jsonataService,
    });
    return {
        homeAssistant,
        jsonataService,
        nodeRedContextService,
        typedInputService,
    };
}
exports.createControllerDependencies = createControllerDependencies;
